@@ -1,23 +1,30 @@
 package Module::Dependency::Indexer;
-
+use strict;
 use File::Find;
+use File::Spec;
 use Storable qw/nstore/;
-use vars qw/$VERSION $UNIFIED @NOINDEX $unified_file/;
+use vars qw/$VERSION $UNIFIED @NOINDEX $unified_file $check_shebang/;
 
-($VERSION) = ('$Revision: 1.8 $' =~ /([\d\.]+)/ );
-@NOINDEX = qw(.AppleDouble /test /CVS/);
+($VERSION) = ('$Revision: 1.14 $' =~ /([\d\.]+)/ );
+@NOINDEX = qw(.AppleDouble /test);
+my %ignore_file_names = map { $_=>1 } qw(
+    CVS
+    .svn
+    .cpan
+);
+$check_shebang = 1;
 
 $unified_file = '/var/tmp/dependence/unified.dat';
 
 sub setIndex {
-	my $file = shift;
+	my $file = _makeAbsolute( shift );
 	TRACE("Trying to set index to <$file>");
 	return unless $file;
 	$unified_file = $file;
 }
 
 sub makeIndex {
-	my @dirs = @_;
+	my @dirs = map { _makeAbsolute( $_ ) } @_;
 	
 	TRACE( "Running search to build indexes" );
 	$UNIFIED = {};
@@ -27,8 +34,26 @@ sub makeIndex {
 	return 1;
 }
 
+sub setShebangCheck {
+	$check_shebang = shift;
+}
+
 ######### PRIVATE
 
+# if we get given relative pathnames then things stop working when File::Find changes working directory
+# the fix is now to ensure we use absolute paths internally.
+sub _makeAbsolute {
+	my $dir = $_[0];
+	if ( File::Spec->file_name_is_absolute( $dir ) ) {
+		TRACE("$dir is an absolute path");
+		return $dir;
+	} else {
+		my $abs = File::Spec->rel2abs( $dir );
+		TRACE("$dir is relative - changed to $abs");
+		return $abs
+	}
+}
+
 sub _storeIndex {
 	TRACE( "storing to disk" );
 	
@@ -66,103 +91,128 @@ sub _wanted {
 	local $_ = $_;
 	my $fname = $File::Find::name;
 	
+        if ($ignore_file_names{$_}) {
+            TRACE("Rejecting $fname");
+            return;
+        }
 	foreach (@NOINDEX) {
-		if (index($fname, $_) > -1) {
-			TRACE("Rejecting $fname");
-			return;
-		}
+            if (index($fname, $_) > -1) {
+                TRACE("Rejecting $fname");
+                return;
+            }
 	}
 	
 	TRACE("Indexing $fname");
+	my $is_perl_script = 0;
 	if (m/\.pm$/) {
 		my $moduleObj = _parseModule( $fname ) || return;
 		$UNIFIED->{'allobjects'}->{ $moduleObj->{'package'} } = $moduleObj;
 	} elsif (m/\.plx?$/) {
-		my $scriptObj = _parseScript( $fname ) || return;
-		push( @{$UNIFIED->{'scripts'}} , $scriptObj->{'package'} );
-		$UNIFIED->{'allobjects'}->{ $scriptObj->{'package'} } = $scriptObj;
+		$is_perl_script++;
+	} elsif ($check_shebang && -f $fname && open(F, "<$fname")) {
+		my $first_line = <F> || '';
+		close F;
+		if ($first_line =~ /^#!.*perl/) {
+			$is_perl_script++;
+		}
 	} else {
 		return;
 	}
+
+	if ($is_perl_script) {
+ 		my $scriptObj = _parseScript( $fname ) || return;
+ 		push( @{$UNIFIED->{'scripts'}} , $scriptObj->{'package'} );
+ 		$UNIFIED->{'allobjects'}->{ $scriptObj->{'package'} } = $scriptObj;
+ 	}
+}
+
+# Get data from a module file, returns a dependency unit object
+sub _parseFile {
+	my $file = shift;
+
+	my $self = {
+		'filename' => $file,
+		'depends_on' => [],
+		'depended_upon_by' => [],
+	};
+	
+	my %seen;
+	
+	# go through the file and try to find out some things
+	local *FILE;
+	open(FILE, $file) or do { warn("Can't open file $file for read: $!"); return undef; };
+	
+	my $in_pod;
+	while(<FILE>) {
+		if ($in_pod) {
+			$in_pod = 0 if /^=cut/;
+			next;
+		}
+
+		# get the package name
+		if (m/^\s*package\s+([\w\:]+)\s*;/) {
+                        # only record the first package seen
+			$self->{'package'} = $1 unless exists $self->{'package'};
+		}
+		# get the dependencies
+		if (m/^\s*use\s+([\w\:]+)/) {
+			push (@{$self->{'depends_on'}}, $1) unless ($seen{$1}++);
+		}
+		# get the dependencies - XXX doesn't deal with require "Foo/Bar.pm"
+		if (m/^\s*require\s+([\w\:]+)/) {
+			push (@{$self->{'depends_on'}}, $1) unless ($seen{$1}++);
+		}
+		
+		# the 'base' pragma - SREZIC
+		if (m/^\s*use\s+base\s+(.*)/) {
+			require Safe;
+			my $safe = new Safe;
+			(my $list = $1) =~ s/\s+\#.*//;
+			$list =~ s/[\r\n]//;
+			while ($list !~ /;\s*$/ && ($_ = <FILE>)) {
+				s/\s+#.*//;
+				s/[\r\n]//;
+				$list .= $_;
+			}
+			$list =~ s/;\s*$//;
+			my(@mods) = $safe->reval($list);
+			foreach my $mod (@mods) {
+				push (@{$self->{'depends_on'}}, $mod) unless ($seen{$mod}++);
+			}
+		}
+
+		$in_pod = 1 if m/^=\w+/ && !m/^=cut/;
+		last if m/^\s*__(END|DATA)__/;
+	}
+	close FILE;
+	
+	return $self;
 }
 
 # Get data from a module file, returns a dependency unit object
 sub _parseModule {
 	my $file = shift;
-	my $self = {
-		'filename' => $file,
-		'package' => '',
-		'depends_on' => [],
-		'depended_upon_by' => [],
-	};
-	my $foundPackage = 0;
-	
-	my %seen;
-	
-	# go through the file and try to find out some things
-	local *FILE;
-	open(FILE, $file) or do { warn("Can't open file $file for read: $!"); return undef; };
-	while(<FILE>) {
-		# get the package name
-		if (m/^\s*package\s+([\w\:]+)\s*;/ && $foundPackage == 0) {
-			$foundPackage = 1;
-			$self->{'package'} = $1;
-			TRACE("*** package is <$1> ***");
-		}
-		# get the dependencies
-		if (m/^\s*use\s+([\w\:]+).*;/) {
-			push (@{$self->{'depends_on'}}, $1) unless ($seen{$1}++);
-		}
-		# get the dependencies
-		if (m/^\s*require\s+([\w\:]+).*;/) {
-			push (@{$self->{'depends_on'}}, $1) unless ($seen{$1}++);
-		}
-		last if m/^__/;
-		last if m/^=\w+/;
-		last if m/^1;/;
-	}
-	close FILE;
-	
-	if ($foundPackage) {
-		return $self;
-	} else {
-		return undef;
+	my $self = _parseFile($file)
+            or return;
+        if (!$self->{'package'}) {
+            warn "No package found in $file\n";
+            return undef;
 	}
+        return $self;
 }
 
 # Get data from a program file, returns a dependency unit object
 sub _parseScript {
 	my $file = shift;
-	
-	my $node = $file;
-	$node =~ s|^.*/||;
-	
-	my $self = {
-		'filename' => $file,
-		'package' => $node,
-		'depends_on' => [],
-	};
-	my $foundPackage = 0;
-	
-	my %seen;
-	
-	# go through the file and try to find out some things
-	local *FILE;
-	open(FILE, $file) or do { warn("Can't open file $file for read: $!"); return undef; };
-	while(<FILE>) {
-		# get the dependencies
-		if (m/\s*use\s+([\w\:]+).*;/) {
-			push (@{$self->{'depends_on'}}, $1) unless ($seen{$1}++);
-		}
-		# get the dependencies
-		if (m/\s*require\s+([\w\:]+).*;/) {
-			push (@{$self->{'depends_on'}}, $1) unless ($seen{$1}++);
-		}
-		last if m/^__/;
-		last if m/^=\w+/;
-	}
-	close FILE;
-	
+	my $self = _parseFile($file)
+            or return;
+
+        # XXX force package for script file to be the filename (without directory)
+	(undef, undef, my $filename) = File::Spec->splitpath( $file );
+        warn "$file contains package $self->{'package'} but we ignore it"
+                if $self->{'package'};
+	$self->{'package'} = $filename;
+
 	return $self;
 }
 
@@ -180,11 +230,14 @@ Module::Dependency::Indexer - creates the databases used by the dependency mappi
 	use Module::Dependency::Indexer;
 	Module::Dependency::Indexer::setIndex( '/var/tmp/dependency/unified.dat' );
 	Module::Dependency::Indexer::makeIndex( $directory, [ $another, $andanother... ] );
+	Module::Dependency::Indexer::setShebangCheck( 0 );
 
 =head1 DESCRIPTION
 
 This module looks at all .pm, .pl and .plx files within and below a given directory/directories 
-(found with File::Find), reads through them and extracts some information about them:
+(found with File::Find), reads through them and extracts some information about them.
+If the shebang check is turned on then it also looks at the first line of all
+other files, to see if they're perl programs too. We extract this information:
 
 =over 4
 
@@ -225,12 +278,17 @@ for datafiles independently of this module.
 
 Default is /var/tmp/dependence/unified.dat
 
-=item makeIndex( $directory, [ $another, $andanother... ] );
+=item makeIndex( $directory, [ $another, $andanother... ] )
 
 Builds, and stores to the current data file, a SINGLE database for all the files found under 
 all of the supplied directories. To create multiple indexes, run this method many times with a setIndex 
 inbetween each so that you don't clobber the previous run's datafile.
 
+=item setShebangCheck( BOOLEAN )
+
+Turns on or off the checking of #! lines for all files that are not .pl, .plx or .pm filenames.
+By default we do check the #! lines.
+
 =back
 
 =head1 NOTE ABOUT WHAT IS INDEXED
@@ -273,6 +331,14 @@ location. Other applications just use that file.
 There is a TRACE stub function, and the module uses TRACE() to log activity. Override our TRACE with your own routine, e.g.
 one that prints to STDERR, to see these messages.
 
+=head1 SEE ALSO
+
+Module::Dependency and the README files.
+
+=head1 VERSION
+
+$Id: Indexer.pm,v 1.13 2002/09/25 23:06:35 piers Exp $
+
 =cut
 
 
